﻿using System.Collections;
using System.Collections.Generic;
using System;
using UnityEngine;
using System.Linq;
using System.Reflection;

namespace Mod
{
    public class Mod : MonoBehaviour
    {
        public static Texture2D Empty = new Texture2D(1, 1);

        public static void Main()
        {
            Mod.Empty.SetPixel(1, 1, Color.clear);
            Mod.Empty.Apply();

            Liquid.Register("REGROWTH", new RegrowthPoison());
            ModAPI.Register(
               new Modification()
               {
                   OriginalItem = ModAPI.FindSpawnable("Empty Syringe"),
                   NameOverride = "Regrowth Syringe",
                   DescriptionOverride = "The contents of the syringe are a mystery. Injecting it into a person will result in the passive ability to regenerate their limbs once torn off. Does not help limbs that are already off.\n(Side Note: Crushed limbs, Chainsaw Stuff, and Bombs Do Not Work With This Method)",
                   CategoryOverride = ModAPI.FindCategory("Biohazard"),
                   ThumbnailOverride = ModAPI.LoadSprite("thumby.png"),
                   AfterSpawn = (Instance) =>
                   {
                       GameObject.Destroy(Instance.GetComponent<EmptySyringe>());
                       var syr = Instance.AddComponent<RegrowthSyringe>();
                   }
               }
            );
        }

        public class RegrowthSyringe : EmptySyringe
        {
            public override string GetLiquidID()
            {
                return "REGROWTH";
            }
        }

        public class RegrowthPoison : TemporaryBodyLiquid
        {
            public RegrowthPoison()
            {
                this.Color = new Color32(95, 239, 160, 255);
            }

            public override void OnEnterContainer(BloodContainer container)
            {
                return;
            }

            public override void OnExitContainer(BloodContainer container)
            {
                return;
            }

            public override void OnEnterLimb(LimbBehaviour limb)
            {
                if (limb.gameObject.GetComponent<Regrower>() == null)
                {
                    limb.gameObject.AddComponent<Regrower>().Limb = limb;
                }
            }

            public const string ID = "REGROWTH";
        }

        public class Regrower : MonoBehaviour
        {
            public LimbBehaviour Limb;
            private List<LimbBehaviour> LimbsToDupe = new List<LimbBehaviour>();
            private float MaxForce;

            private void Start()
            {
                if (this.Limb.HasJoint)
                {
                    this.MaxForce = this.Limb.Joint.breakForce;
                    Debug.Log(this.Limb.Joint.breakForce);
                    this.Limb.Joint.breakForce = Mathf.Infinity;
                    this.Limb.Joint.breakTorque = Mathf.Infinity;
                    this.Limb.BreakingThreshold = Mathf.Infinity;
                }
            }

            public void FixedUpdate()
            {
                if (this.Limb.HasJoint && this.Limb.RoughClassification != LimbBehaviour.BodyPart.Head && this.Limb.gameObject.GetComponent<DontRegen>() == null)
                {
                    if (this.Limb.Joint.GetReactionForce(Time.fixedDeltaTime).magnitude > this.MaxForce)
                    {
                        ObliterateLimbs();
                    }
                }
            }

            private IEnumerator ConvertToSandAndDoWackyStuff(LimbBehaviour limb)
            {
                Texture2D oldTex = (Texture2D)limb.PhysicalBehaviour.spriteRenderer.material.GetTexture("_BoneTex");
                limb.Collider.isTrigger = true;
                limb.ImmuneToDamage = true;
                limb.CirculationBehaviour.ImmuneToDamage = true;
                //limb.PhysicalBehaviour.MakeWeightless();
                limb.PhysicalBehaviour.rigidbody.mass = 0f;
                limb.PhysicalBehaviour.spriteRenderer.material.SetTexture("_BoneTex", Mod.Empty);
                limb.SkinMaterialHandler.AcidProgress = 1f;
                limb.gameObject.AddComponent<DontRegen>();
                if (limb.HasJoint && limb == this.Limb)
                {
                    var globalPosition = limb.transform.TransformPoint(limb.Joint.anchor);
                    limb.SkinMaterialHandler.ShotDamage(15f, globalPosition, false);
                    limb.SkinMaterialHandler.ImpactDamage(15f, globalPosition, false);
                    limb.CirculationBehaviour.BleedingRate += 10f;
                    limb.Joint.attachedRigidbody.gameObject.GetComponent<SkinMaterialHandler>().ShotDamage(15f, globalPosition, true);
                    limb.Joint.attachedRigidbody.gameObject.GetComponent<SkinMaterialHandler>().ImpactDamage(15f, globalPosition, true);
                    limb.Joint.attachedRigidbody.gameObject.GetComponent<CirculationBehaviour>().CreateBleedingParticle(globalPosition, Vector2.down);
                    limb.Joint.attachedRigidbody.gameObject.GetComponent<CirculationBehaviour>().BleedingRate += 10f;
                    limb.PhysicalBehaviour.PlayClipOnce(limb.Person.DismembermentClips.PickRandom<AudioClip>());
                }
                yield return new WaitForSeconds(1f);
                while (limb.SkinMaterialHandler.AcidProgress > 0.65f)
                {
                    limb.CirculationBehaviour.AddLiquid(Liquid.GetLiquid("LIFE SERUM"), 0.25f);
                    yield return new WaitForSeconds(0.25f);
                }
                limb.Collider.isTrigger = false;
                limb.PhysicalBehaviour.rigidbody.mass = limb.PhysicalBehaviour.InitialMass;
                limb.PhysicalBehaviour.spriteRenderer.material.SetTexture("_BoneTex", oldTex);
                GameObject.Destroy(limb.gameObject.GetComponent<DontRegen>());
                //limb.PhysicalBehaviour.MakeWeightful();
                limb.ImmuneToDamage = false;
                limb.Health = limb.InitialHealth;
                limb.CirculationBehaviour.ImmuneToDamage = false;
            }

            private IEnumerator DupeLimbs()
            {
                bool Grab = false;
                GameObject Holding = null;
                Vector2 con = Vector2.zero;
                Dictionary<string, LimbBehaviour> Duped = new Dictionary<string, LimbBehaviour>();
                var tool = GameObject.FindObjectOfType<DragTool>();
                if (this.LimbsToDupe.Count > 0)
                {
                    if (tool.ActiveSingleSelected.GetComponent<LimbBehaviour>() != null && this.LimbsToDupe.Contains(GameObject.FindObjectOfType<DragTool>().ActiveSingleSelected.gameObject.GetComponent<LimbBehaviour>()))
                    {
                        Grab = true;
                        Holding = tool.ActiveSingleSelected.gameObject;
                        con = tool.gameObject.GetComponent<HingeJoint2D>().connectedAnchor;
                        tool.OnDeselect();
                    }
                    foreach (var limb in this.LimbsToDupe)
                    {
                        if (limb.HasJoint)
                        {
                            JointInformation thing = limb.gameObject.AddComponent<JointInformation>();
                            thing.Anchor = limb.Joint.anchor;
                            thing.ConnectedAnchor = limb.Joint.connectedAnchor;
                            thing.UpperLimit = limb.Joint.limits.max;
                            thing.LowerLimit = limb.Joint.limits.min;
                            thing.ConnectedLimb = limb.Joint.connectedBody.gameObject.name;
                        }
                    }
                    foreach (var limb in this.LimbsToDupe)
                    {
                        LimbBehaviour newLimb = GameObject.Instantiate<GameObject>(limb.gameObject, null, true).GetComponent<LimbBehaviour>();
                        newLimb.transform.position = limb.transform.position;
                        newLimb.transform.rotation = limb.transform.rotation;
                        newLimb.PhysicalBehaviour.rigidbody.velocity = limb.PhysicalBehaviour.rigidbody.velocity;
                        newLimb.PhysicalBehaviour.rigidbody.angularVelocity = limb.PhysicalBehaviour.rigidbody.angularVelocity;
                        if (limb.gameObject.GetComponent<WireBehaviour>() != null)
                        {
                            GameObject.Destroy(limb.gameObject.GetComponent<WireBehaviour>().untypedJoint);
                            GameObject.Destroy(limb.gameObject.GetComponent<WireBehaviour>());
                        }
                        limb.Person.Limbs = limb.Person.Limbs.Concat(new LimbBehaviour[1] { newLimb }).ToArray();
                        newLimb.PhysicalBehaviour.SpawnSpawnParticles = false;
                        foreach (KeyValuePair<PoseState, RagdollPose> keyValuePair in newLimb.Person.LinkedPoses)
                        {
                            var animstuff = new RagdollPose.LimbPose(newLimb.GetComponent<LimbBehaviour>(), 45f + UnityEngine.Random.Range(-25f, 25f));
                            keyValuePair.Value.AngleDictionary.Add(newLimb.GetComponent<LimbBehaviour>(), animstuff);
                            keyValuePair.Value.Angles.Add(animstuff);
                            keyValuePair.Value.ConstructDictionary();
                        }
                        newLimb.PhysicalBehaviour.spriteRenderer.sprite = limb.PhysicalBehaviour.spriteRenderer.sprite;
                        newLimb.PhysicalBehaviour.spriteRenderer.material = limb.PhysicalBehaviour.spriteRenderer.material;
                        newLimb.PhysicalBehaviour.spriteRenderer.maskInteraction = limb.PhysicalBehaviour.spriteRenderer.maskInteraction; //jic
                        newLimb.gameObject.FixColliders();
                        var col = limb.gameObject.AddComponent<NoCollide>();
                        col.NoCollideSetA = limb.Person.GetComponentsInChildren<Collider2D>();
                        col.NoCollideSetB = newLimb.gameObject.GetComponentsInChildren<Collider2D>();
                        newLimb.NodeBehaviour.DisconnectFromEverything();
                        if (limb.HasJoint)
                        {
                            GameObject.Destroy(newLimb.Joint);
                            newLimb.HasJoint = false;
                        }
                        newLimb.PhysicalBehaviour.rigidbody.mass = newLimb.PhysicalBehaviour.InitialMass;
                        newLimb.SkinMaterialHandler.bulletHolePoints = limb.SkinMaterialHandler.bulletHolePoints;
                        newLimb.SkinMaterialHandler.bulletHolePointIndex = limb.SkinMaterialHandler.bulletHolePointIndex;
                        newLimb.SkinMaterialHandler.damagePointIndex = limb.SkinMaterialHandler.damagePointIndex;
                        newLimb.SkinMaterialHandler.damagePoints = limb.SkinMaterialHandler.damagePoints;
                        newLimb.gameObject.name = limb.gameObject.name;
                        newLimb.SkinMaterialHandler.AcidProgress = 0f;
                        newLimb.SkinMaterialHandler.RottenProgress = limb.SkinMaterialHandler.RottenProgress;
                        newLimb.SkinMaterialHandler.Sync();
                        newLimb.SkinMaterialHandler.renderer.material.SetFloat("_AcidProgress", 0f);
                        newLimb.SkinMaterialHandler.renderer.material.SetFloat("_BurnProgress", limb.PhysicalBehaviour.BurnProgress);
                        newLimb.SkinMaterialHandler.renderer.material.SetFloat("_RottenProgress", limb.SkinMaterialHandler.RottenProgress);
                        typeof(LimbBehaviour).GetTypeInfo().GetDeclaredMethod("CreateContextMenuOptions").Invoke(newLimb, new object[0]);
                        newLimb.PhysicalBehaviour.SimulateTemperature = false;
                        newLimb.PhysicalBehaviour.BurningProgressionMultiplier = 0f;
                        Duped.Add(newLimb.gameObject.name, newLimb);
                        if (Grab && limb.gameObject == Holding)
                        {
                            SelectionController.Main.ClearSelection();
                            tool.ActiveSingleSelected = newLimb.PhysicalBehaviour;
                            tool.OnSelect();
                            tool.StartCoroutine(SetCon(limb, tool, con));
                        }
                    }
                    foreach (var keylimb in Duped)
                    {
                        LimbBehaviour limb = keylimb.Value;
                        FreezeBehaviour fez;
                        JointInformation info;
                        if (limb != null)
                        {
                            if (limb.gameObject.TryGetComponent<FreezeBehaviour>(out fez) == true) //== true is added just in case
                            {
                                FreezeStackController.RequestFreeze(limb.PhysicalBehaviour.rigidbody);
                                UnityEngine.Object.Destroy(fez);
                                limb.PhysicalBehaviour.rigidbody.bodyType = RigidbodyType2D.Dynamic;
                            }
                            limb.gameObject.layer = 10;
                            if (limb.TryGetComponent<JointInformation>(out info))
                            {
                                if (Duped.ContainsKey(info.ConnectedLimb))
                                {
                                    var joint = limb.gameObject.AddComponent<HingeJoint2D>();
                                    joint.connectedBody = Duped[info.ConnectedLimb].PhysicalBehaviour.rigidbody;
                                    joint.anchor = info.Anchor;
                                    joint.autoConfigureConnectedAnchor = false;
                                    joint.connectedAnchor = info.ConnectedAnchor;
                                    joint.limits = new JointAngleLimits2D()
                                    {
                                        max = info.UpperLimit,
                                        min = info.LowerLimit
                                    };
                                    joint.useMotor = false;
                                    joint.motor = new JointMotor2D()
                                    {
                                        maxMotorTorque = limb.MotorStrength,
                                        motorSpeed = 0f
                                    };
                                    limb.Joint = joint;
                                    limb.HasJoint = true;
                                    limb.BaseStrength = 0f;
                                    limb.BloodMuscleStrengthRatio = 0f;
                                    limb.gameObject.AddComponent<HingeJointLimitAutofixBehaviour>();
                                }
                            }
                        }
                    }
                }
                this.LimbsToDupe.Clear();
                Duped.Clear();
                yield return new WaitForEndOfFrame();
            }

            private IEnumerator SetCon(LimbBehaviour limb, DragTool tool, Vector2 con)
            {
                yield return new WaitForSeconds(0.01f);
                tool.gameObject.GetComponent<HingeJoint2D>().connectedAnchor = con;
            }

            private void ObliterateLimbs()
            {
                foreach (var limb in this.Limb.Person.Limbs)
                {
                    if (limb == this.Limb || limb.gameObject.GetComponent<DontRegen>() != null)
                    {
                        continue;
                    }
                    if (limb.DistanceToBrain > this.Limb.DistanceToBrain && limb.NodeBehaviour.IsConnectedToRoot)
                    {
                        if (this.Limb.RoughClassification == LimbBehaviour.BodyPart.Legs)
                        {
                            if (limb.RoughClassification == LimbBehaviour.BodyPart.Legs)
                            {
                                if (this.Limb.gameObject.name.Contains("Front")) //This part is stupid and dumb. Wish the nodeconnection behaviour had a function to check if a limb is attached to a limb that isn't next to it.
                                {
                                    if (limb.gameObject.name.Contains("Front"))
                                    {
                                        this.LimbsToDupe.Add(limb);
                                        limb.StartCoroutine(ConvertToSandAndDoWackyStuff(limb));
                                    }
                                }
                                else
                                {
                                    if (!limb.gameObject.name.Contains("Front"))
                                    {
                                        this.LimbsToDupe.Add(limb);
                                        limb.StartCoroutine(ConvertToSandAndDoWackyStuff(limb));
                                    }
                                }
                            }
                        }
                        if (this.Limb.RoughClassification == LimbBehaviour.BodyPart.Arms)
                        {
                            if (limb.RoughClassification == LimbBehaviour.BodyPart.Arms)
                            {
                                if (this.Limb.gameObject.name.Contains("Front")) //This part is stupid and dumb. Wish the nodeconnection behaviour had a function to check if a limb is attached to a limb that isn't next to it.
                                {
                                    if (limb.gameObject.name.Contains("Front"))
                                    {
                                        this.LimbsToDupe.Add(limb);
                                        limb.StartCoroutine(ConvertToSandAndDoWackyStuff(limb));
                                    }
                                }
                                else
                                {
                                    if (!limb.gameObject.name.Contains("Front"))
                                    {
                                        this.LimbsToDupe.Add(limb);
                                        limb.StartCoroutine(ConvertToSandAndDoWackyStuff(limb));
                                    }
                                }
                            }
                        }
                        else
                        {
                            this.LimbsToDupe.Add(limb);
                            limb.StartCoroutine(ConvertToSandAndDoWackyStuff(limb));
                        }
                    }
                }
                if (this.Limb.NodeBehaviour.IsConnectedToRoot && this.Limb.gameObject.GetComponent<DontRegen>() == null)
                {
                    this.LimbsToDupe.Add(this.Limb);
                    this.Limb.StartCoroutine(ConvertToSandAndDoWackyStuff(this.Limb));
                    this.Limb.StartCoroutine(DupeLimbs());
                }
            }
        }

        public class JointInformation : MonoBehaviour
        {
            public Vector2 Anchor;
            public Vector2 ConnectedAnchor;
            public float UpperLimit;
            public float LowerLimit;
            public string ConnectedLimb;
        }

        public class DontRegen : MonoBehaviour {}
    }
}